// Cubic FM Radio - Battery Charger Firmware / for ATtiny13A
// 2013.9.27 Eleken
// rev_c1815@yahoo.co.jp
// http://www1.adachi.ne.jp/2sc1815/electronics/

// main clock : 1.2MHz
// Fuse bits  : high 0b11111101 (enable BOD reset)

#include "main.h"

/* Functions */

/* Grobals */
enum tState{
	NOCHARGE, 
	TRICKLE1,
	CHARGING,
	TRICKLE2,
	POSTCHARGE
};
volatile enum tState g_cState;

/* Timer */
static void InitTimer(void);
uint16_t DoADC(void);

ISR(ADC_vect){
}
// timer0 : about 220 ms per interrupt
ISR(TIM0_OVF_vect){
	static uint16_t subcnt1 = 0;
	static uint8_t subcnt2 = 0;
	// charge timer; 8 hour = 28800 sec. = 130,000 counts
	uint16_t vbat = DoADC();
	
	if(g_cState == CHARGING){
		if((vbat < TRICKLE_MIN) || (vbat > TRICKLE_MAX)) g_cState = NOCHARGE;
		else if(subcnt2 > 130) g_cState = TRICKLE2;
	}else if((g_cState == POSTCHARGE) || (g_cState == TRICKLE2)){
		if((vbat >= TRICKLE_MIN) && (vbat <= TRICKLE_MAX)) g_cState = TRICKLE2;
		else g_cState = POSTCHARGE;
	}else{
		if((vbat >= CHARGE_MIN) && (vbat <= CHARGE_MAX)) {
			subcnt1 = subcnt2 = 0;
			g_cState = CHARGING;
		}
		else if((vbat >= TRICKLE_MIN) && (vbat <= TRICKLE_MAX)) {
			g_cState = TRICKLE1;
		}
		else{
			g_cState = NOCHARGE;
		}
	}
	
	if(subcnt1++ >= 1000){
		subcnt1 = 0;
		subcnt2 ++;
	}
	
}


static void InitTimer(void){
	// timer0 (220ms)
	TCCR0A = 0b00000000; // 
	TCCR0B = 0b00000101; // clk/1024
	
	OCR0A  = 0; // PWM value
	TIMSK0 = 0b00000010; // overflow interrput enable
	TCNT0  = 0;
	
	
}

// Initialize
static uint8_t Init(void){

	cli();
	
	g_cState = NOCHARGE;
	
	// init clock	
	
	// init pins
	PORTB = 0b00000000;
	DDRB  = 0b00000001;
	
	
	// init adc
	ADMUX  = 0b00000010; // channel : ADC2, Vref = 5.0V
	ADCSRA = 0b10001011; // CK/8 = 150kHz
	ADCSRB = 0b00000000;
	DIDR0  = 0b00010000;
	
	
	// init timer
	InitTimer();
	
	sei();
	
	return 0;
	
}

// ret: battery voltage [mv]
uint16_t DoADC(void){
	uint8_t DDRB_old = DDRB;
	if((DDRB & 0b00001100) == 0) DDRB |= 0b00000100;
	
	
	_delay_ms(1);
	
	ADCSRA |= 0b01000000;
	while(ADCSRA & 0b01000000);
	uint16_t temp = ADC;
	
	DDRB = DDRB_old;
	
	
	// V = ADC*1.1*1000/1024*2
//	temp = ((uint32_t)temp * 275) / 128;
	// V = ADC*5.0*1000/1024*2
	temp = ((uint32_t)temp * 1250) / 128;
	if(temp > 400) temp -= 400;
	else temp = 0;
	return temp;
	
}

int main(void){
	
	// 
	if(Init()){
		return -1;
	}
	
	while(1){
		uint8_t i;
		if((g_cState == TRICKLE1) || (g_cState == TRICKLE2)){
			SBI(DDRB,  PB2);
			CBI(DDRB,  PB3);
			SBI(PORTB, PB0);
			_delay_ms(100);
			CBI(PORTB, PB0);
			for(i=0; i<9; i++) _delay_ms(100);
		}else if(g_cState == CHARGING){
			CBI(DDRB,  PB2);
			SBI(DDRB,  PB3);
			SBI(PORTB, PB0);
			for(i=0; i<10; i++) _delay_ms(100);
		}else{
			CBI(DDRB,  PB2);
			CBI(DDRB,  PB3);
			CBI(PORTB, PB0);
			for(i=0; i<10; i++) _delay_ms(100);
		}
	}
	while(1);
	

	return 0;
}
